package javax.swing;

import javax.accessibility.AccessibleState;
import javax.accessibility.AccessibleStateSet;
import javax.swing.event.ChangeEvent;
import java.io.Serializable;
import javax.swing.event.ChangeListener;
import java.beans.PropertyChangeEvent;
import javax.accessibility.AccessibleContext;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.awt.Rectangle;
import javax.swing.text.TextAction;
import java.awt.AWTEvent;
import java.awt.event.ActionEvent;
import java.awt.event.InputEvent;
import java.awt.EventQueue;
import java.awt.event.ActionListener;
import java.awt.Font;
import java.awt.Insets;
import java.awt.Dimension;
import javax.swing.text.PlainDocument;
import javax.swing.text.Document;
import java.beans.PropertyChangeListener;
import javax.swing.text.JTextComponent;

public class JTextField extends JTextComponent implements SwingConstants
{
    private Action action;
    private PropertyChangeListener actionPropertyChangeListener;
    public static final String notifyAction = "notify-field-accept";
    private BoundedRangeModel visibility;
    private int horizontalAlignment;
    private int columns;
    private int columnWidth;
    private String command;
    private static final Action[] defaultActions;
    private static final String uiClassID = "TextFieldUI";
    
    public JTextField() {
        this(null, null, 0);
    }
    
    public JTextField(String s) {
        this(null, s, 0);
    }
    
    public JTextField(int n) {
        this(null, null, n);
    }
    
    public JTextField(String s, int n) {
        this(null, s, n);
    }
    
    public JTextField(Document defaultModel, String text, int columns) {
        this.horizontalAlignment = 10;
        if (columns < 0) {
            throw new IllegalArgumentException("columns less than zero.");
        }
        (this.visibility = new DefaultBoundedRangeModel()).addChangeListener(new JTextField.ScrollRepainter());
        this.columns = columns;
        if (defaultModel == null) {
            defaultModel = this.createDefaultModel();
        }
        this.setDocument(defaultModel);
        if (text != null) {
            this.setText(text);
        }
        return;
    }
    
    @Override
    public String getUIClassID() {
        return "TextFieldUI";
    }
    
    @Override
    public void setDocument(Document document) {
        if (document != null) {
            document.putProperty("filterNewlines", Boolean.TRUE);
        }
        super.setDocument(document);
    }
    
    @Override
    public boolean isValidateRoot() {
        return !(SwingUtilities.getUnwrappedParent(this) instanceof JViewport);
    }
    
    public int getHorizontalAlignment() {
        return this.horizontalAlignment;
    }
    
    public void setHorizontalAlignment(int horizontalAlignment) {
        if (horizontalAlignment == this.horizontalAlignment) {
            return;
        }
        int horizontalAlignment2 = this.horizontalAlignment;
        if (horizontalAlignment == 2 || horizontalAlignment == 0 || horizontalAlignment == 4 || horizontalAlignment == 10 || horizontalAlignment == 11) {
            this.firePropertyChange("horizontalAlignment", horizontalAlignment2, this.horizontalAlignment = horizontalAlignment);
            this.invalidate();
            this.repaint();
            return;
        }
        throw new IllegalArgumentException("horizontalAlignment");
    }
    
    protected Document createDefaultModel() {
        return new PlainDocument();
    }
    
    public int getColumns() {
        return this.columns;
    }
    
    public void setColumns(int columns) {
        int columns2 = this.columns;
        if (columns < 0) {
            throw new IllegalArgumentException("columns less than zero.");
        }
        if (columns != columns2) {
            this.columns = columns;
            this.invalidate();
        }
        return;
    }
    
    protected int getColumnWidth() {
        if (this.columnWidth == 0) {
            this.columnWidth = this.getFontMetrics(this.getFont()).charWidth('m');
        }
        return this.columnWidth;
    }
    
    @Override
    public Dimension getPreferredSize() {
        Dimension preferredSize = super.getPreferredSize();
        if (this.columns != 0) {
            Insets insets = this.getInsets();
            preferredSize.width = this.columns * this.getColumnWidth() + insets.left + insets.right;
            return preferredSize;
        }
        return preferredSize;
    }
    
    @Override
    public void setFont(Font font) {
        super.setFont(font);
        this.columnWidth = 0;
    }
    
    public synchronized void addActionListener(ActionListener l) {
        this.listenerList.add(ActionListener.class, l);
    }
    
    public synchronized void removeActionListener(ActionListener l) {
        if (l != null && this.getAction() == l) {
            this.setAction(null);
        }
        else {
            this.listenerList.remove(ActionListener.class, l);
        }
    }
    
    public synchronized ActionListener[] getActionListeners() {
        return (ActionListener[])this.listenerList.getListeners(ActionListener.class);
    }
    
    protected void fireActionPerformed() {
        Object[] listenerList = this.listenerList.getListenerList();
        int modifiers = 0;
        AWTEvent currentEvent = EventQueue.getCurrentEvent();
        if ((currentEvent instanceof InputEvent)) {
            modifiers = ((InputEvent)currentEvent).getModifiers();
        }
        else if ((currentEvent instanceof ActionEvent)) {
            modifiers = ((ActionEvent)currentEvent).getModifiers();
        }
        ActionEvent o = new ActionEvent(this, 1001, (this.command != null) ? this.command : this.getText(), EventQueue.getMostRecentEventTime(), modifiers);
        for (int i = listenerList.length - 2; i >= 0; i -= 2) {
            if (listenerList[i] == ActionListener.class) {
                ((ActionListener)listenerList[i + 1]).actionPerformed(o);
            }
        }
    }
    
    public void setActionCommand(String command) {
        this.command = command;
    }
    
    public void setAction(Action action) {
        Action action2 = this.getAction();
        if (this.action == null || !this.action.equals(action)) {
            this.action = action;
            if (action2 != null) {
                this.removeActionListener(action2);
                action2.removePropertyChangeListener(this.actionPropertyChangeListener);
                this.actionPropertyChangeListener = null;
            }
            this.configurePropertiesFromAction(this.action);
            if (this.action != null) {
                if (!this.isListener(ActionListener.class, this.action)) {
                    this.addActionListener(this.action);
                }
                this.actionPropertyChangeListener = this.createActionPropertyChangeListener(this.action);
                this.action.addPropertyChangeListener(this.actionPropertyChangeListener);
            }
            this.firePropertyChange("action", action2, this.action);
        }
    }
    
    private boolean isListener(Class clazz, ActionListener actionListener) {
        boolean n = false;
        Object[] listenerList = this.listenerList.getListenerList();
        for (int i = listenerList.length - 2; i >= 0; i -= 2) {
            if (listenerList[i] == clazz && listenerList[i + 1] == actionListener) {
                n = true;
            }
        }
        return n;
    }
    
    public Action getAction() {
        return this.action;
    }
    
    protected void configurePropertiesFromAction(Action actionCommandFromAction) {
        AbstractAction.setEnabledFromAction(this, actionCommandFromAction);
        AbstractAction.setToolTipTextFromAction(this, actionCommandFromAction);
        this.setActionCommandFromAction(actionCommandFromAction);
    }
    
    protected void actionPropertyChanged(Action actionCommandFromAction, String s) {
        if (s == "ActionCommandKey") {
            this.setActionCommandFromAction(actionCommandFromAction);
        }
        else if (s == "enabled") {
            AbstractAction.setEnabledFromAction(this, actionCommandFromAction);
        }
        else if (s == "ShortDescription") {
            AbstractAction.setToolTipTextFromAction(this, actionCommandFromAction);
        }
    }
    
    private void setActionCommandFromAction(Action action) {
        this.setActionCommand((action == null) ? null : ((String)action.getValue("ActionCommandKey")));
    }
    
    protected PropertyChangeListener createActionPropertyChangeListener(Action action) {
        return new JTextField.TextFieldActionPropertyChangeListener(this, action);
    }
    
    @Override
    public Action[] getActions() {
        return TextAction.augmentList(super.getActions(), JTextField.defaultActions);
    }
    
    public void postActionEvent() {
        this.fireActionPerformed();
    }
    
    public BoundedRangeModel getHorizontalVisibility() {
        return this.visibility;
    }
    
    public int getScrollOffset() {
        return this.visibility.getValue();
    }
    
    public void setScrollOffset(int value) {
        this.visibility.setValue(value);
    }
    
    @Override
    public void scrollRectToVisible(Rectangle rectangle) {
        int value = rectangle.x + this.visibility.getValue() - this.getInsets().left;
        int n = value + rectangle.width;
        if (value < this.visibility.getValue()) {
            this.visibility.setValue(value);
        }
        else if (n > this.visibility.getValue() + this.visibility.getExtent()) {
            this.visibility.setValue(n - this.visibility.getExtent());
        }
    }
    
    boolean hasActionListener() {
        Object[] listenerList = this.listenerList.getListenerList();
        int i = listenerList.length - 2;
        while (i >= 0) {
            if (listenerList[i] == ActionListener.class) {
                return true;
            }
            i -= 2;
        }
        return false;
    }
    
    private void writeObject(ObjectOutputStream objectOutputStream) throws IOException {
        objectOutputStream.defaultWriteObject();
        if (this.getUIClassID().equals("TextFieldUI")) {
            int n = (byte)(JComponent.getWriteObjCounter(this) - 1);
            JComponent.setWriteObjCounter(this, (byte)n);
            if (n == 0 && this.ui != null) {
                this.ui.installUI(this);
            }
        }
    }
    
    @Override
    protected String paramString() {
        String str;
        if (this.horizontalAlignment == 2) {
            str = "LEFT";
        }
        else if (this.horizontalAlignment == 0) {
            str = "CENTER";
        }
        else if (this.horizontalAlignment == 4) {
            str = "RIGHT";
        }
        else if (this.horizontalAlignment == 10) {
            str = "LEADING";
        }
        else if (this.horizontalAlignment == 11) {
            str = "TRAILING";
        }
        else {
            str = "";
        }
        return super.paramString() + ",columns=" + this.columns + ",columnWidth=" + this.columnWidth + ",command=" + ((this.command != null) ? this.command : "") + ",horizontalAlignment=" + str;
    }
    
    @Override
    public AccessibleContext getAccessibleContext() {
        if (this.accessibleContext == null) {
            this.accessibleContext = new JTextField.AccessibleJTextField();
        }
        return this.accessibleContext;
    }
    
    static {
        defaultActions = new Action[] { new JTextField.NotifyAction() };
    }
    
    private static class TextFieldActionPropertyChangeListener extends ActionPropertyChangeListener<JTextField>
    {
        TextFieldActionPropertyChangeListener(JTextField textField, Action action) {
            super(textField, action);
        }
        
        @Override
        protected void actionPropertyChanged(JTextField textField, Action action, PropertyChangeEvent propertyChangeEvent) {
            if (AbstractAction.shouldReconfigure(propertyChangeEvent)) {
                textField.configurePropertiesFromAction(action);
            }
            else {
                textField.actionPropertyChanged(action, propertyChangeEvent.getPropertyName());
            }
        }
    }
    
    static class NotifyAction extends TextAction
    {
        NotifyAction() {
            super("notify-field-accept");
        }
        
        @Override
        public void actionPerformed(ActionEvent actionEvent) {
            JTextComponent focusedComponent = this.getFocusedComponent();
            if ((focusedComponent instanceof JTextField)) {
                ((JTextField)focusedComponent).postActionEvent();
            }
        }
        
        @Override
        public boolean isEnabled() {
            JTextComponent focusedComponent = this.getFocusedComponent();
            return focusedComponent instanceof JTextField && ((JTextField)focusedComponent).hasActionListener();
        }
    }
    
    class ScrollRepainter implements ChangeListener, Serializable
    {
        @Override
        public void stateChanged(ChangeEvent changeEvent) {
            JTextField.this.repaint();
        }
    }
    
    protected class AccessibleJTextField extends JTextComponent.AccessibleJTextComponent
    {
        @Override
        public AccessibleStateSet getAccessibleStateSet() {
            AccessibleStateSet accessibleStateSet = super.getAccessibleStateSet();
            accessibleStateSet.add(AccessibleState.SINGLE_LINE);
            return accessibleStateSet;
        }
    }
}
